#include "app_timer.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_crypto_init.h"
#include "nrfx_clock.h"

#include "nrf_delay.h"

#include "customble.h"

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

void timers_init(void)
{
  ret_code_t err_code = app_timer_init();
  APP_ERROR_CHECK(err_code);
}

static void idle_state_handle(void)
{
  while (!NRF_LOG_PROCESS())
  {
    // repeat until it returns false (means no more log entries)
  }
  nrf_pwr_mgmt_run();
}

/**@brief Function for initializing power management.
 */
void power_management_init(void)
{
  ret_code_t err_code;
  err_code = nrf_pwr_mgmt_init();
  APP_ERROR_CHECK(err_code);
}

/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}

void check_clock_source(void)
{
  NRF_LOG_INFO("checking LFCLK ...");
  NRF_CLOCK->TASKS_LFCLKSTART = 1;
  while (NRF_CLOCK->EVENTS_LFCLKSTARTED == false)
  {
    // wait...
  }
  NRF_LOG_INFO("LFCLK started!");
  NRF_LOG_FLUSH();
}

void print_my_addr(void)
{
    ble_gap_addr_t my_addr;
    ret_code_t err_code = sd_ble_gap_addr_get(&my_addr);
    APP_ERROR_CHECK(err_code);
    NRF_LOG_INFO("my addr: %02x:%02x:%02x:%02x:%02x:%02x", my_addr.addr[5],
                                                            my_addr.addr[4],
                                                            my_addr.addr[3],
                                                            my_addr.addr[2],
                                                            my_addr.addr[1],
                                                            my_addr.addr[0]);
    NRF_LOG_FLUSH();

}

void check_definitions(void)
{
    NRF_LOG_INFO("NRFX_RNG_ENABLED %d", NRFX_RNG_ENABLED);
    NRF_LOG_INFO("CLOCK_CONFIG_LF_SRC %d", CLOCK_CONFIG_LF_SRC);
    NRF_LOG_INFO("NRF_SDH_CLOCK_LF_SRC %d", NRF_SDH_CLOCK_LF_SRC);
    NRF_LOG_INFO("NRF_SDH_CLOCK_LF_RC_CTIV %d", NRF_SDH_CLOCK_LF_RC_CTIV);
    NRF_LOG_INFO("NRF_SDH_CLOCK_LF_RC_TEMP_CTIV %d", NRF_SDH_CLOCK_LF_RC_TEMP_CTIV);
    NRF_LOG_INFO("NRF_SDH_CLOCK_LF_ACCURACY %d", NRF_SDH_CLOCK_LF_ACCURACY);
    NRF_LOG_FLUSH();
}

int main(void)
{
  log_init();
  check_definitions();
  check_clock_source();
  timers_init();
  ble_timers_init();
  ble_stack_init();

  power_management_init();
  
  print_my_addr();

  advertising_init();

  advertising_start();

  for (;;)
  {
    idle_state_handle();
    //sd_app_evt_wait();
  }
}
